"""
Provides different functions needed for the QMC and resolution method.
"""
import pstats, os
from random import random as myrandom
from random import randint as myrandint

def cube_distance(x,y, modus='normal'):
    """ Calculate cube distance
        
        The modus qmc also considers "don't care" for the cube distance.
        
        @param x: cube 1
        @type x: string
        
        @param y: cube 2
        @type y: string
        
        @param modus: normal (def) or qmc
        @type string: string
    """
    if len(x) != len(y):
        raise IOError, 'cubes must have same dimension'
    if modus == 'qmc':
        return [literals[0] == literals[1] for literals in zip(x,y)].count(False)
    else:
        return [(literals[0] == literals[1]) or ('*' in literals) for literals in zip(x,y)].count(False)

def resolvent(x,y, mode='normal'):
    """ Generate the resolvent (only QMC) of cubes x and y.

        @param x: cube 1
        @type x: string
        
        @param y: cube 2
        @type y: string

        @return: string
    """
    output = []
    assert len(x) == len(y)
    if cube_distance(x,y) != 1:
        return ''
    if mode == 'qmc':
        for literal in zip(x,y):
            if literal[0] == literal[1]:
                output.append(literal[0])
            else:
                output.append('*')
    elif mode == 'normal':
        for literal in zip(x,y):
            if literal[0] == literal[1]:
                output.append(literal[0])
            elif list(literal).count('*') == 1:
                # sorted -> ['*', '1'] or ['*', '0']
                output.append(sorted(literal)[-1])
            else:
                output.append('*')
    else:
        raise ValueError, "mode %s unknown for resolvent" % mode
    return ''.join(output)


def create_sop(terms, dimension=3, likelyhood={'0':0.4, '*':0.1, '1':0.5}):
    """ Generate random SOP onset
    """
    assert type(terms) == int
    onset = set()
    while len(onset) < terms:
        term = []
        for j in range(dimension):
            tmp = myrandom()
            if tmp < likelyhood['0']:
                term.append('0')
            elif  tmp < likelyhood['*'] + likelyhood['0']:
                term.append('*')
            else:
                term.append('1')
        onset.add(''.join(term))
    return onset
        

def create_truthtable(dimension=3, coverage=0.4, bound=10000):
    """ Generate random onset.

        @param dimension: of Boolean space
        @type x: positive integer
        @param coverage: ratio of onset minterms to all minterms [0..1]
        @type y: float
        @param bound: upper bound of trial to avoid endless loops.
        @type y: integer
        
        
    """
    onset = []
    num_minterms = int(coverage*2**dimension)
    iteration = 0
    while iteration < bound and len(onset) < num_minterms:
        iteration += 1
        try:
           tmp = list(myrandint(2, size=dimension))
        except:
           tmp = [myrandint(0, 1) for x in range(dimension)]
        if tmp not in onset:
            onset.append(tmp)
    
    onset = [''.join(map(str, minterm)) for minterm in onset]
    return onset

def cube2minterms(inputcube):
    """ Minterms covered by a cube
    """
    assert type(inputcube) == str
    #   Nothing else than dont care 1 and 0
    assert len(set(inputcube).difference(['*', '1', '0'])) == 0

    minterms = [inputcube]
    cubes = [cube for cube in minterms if '*' in cube]
    while cubes != []:
        for cube in cubes:
            minterms.remove(cube)
            pos = cube.index('*')
            minterms.append( cube[:pos] + '0' + cube[pos+1:] )
            minterms.append( cube[:pos] + '1' + cube[pos+1:] )
        cubes = [cube for cube in minterms if '*' in cube]
    return minterms

def int2bin(input, dim):
    """ Convert integer to binary string

        @param input: integer of interest
        @type input: integer
        @param dim: Dimension of Boolean space (prepended zeros)
        @type dim: integer
        
    """
    out=''
    index=dim-1
    number=input
    while index > -1:
        new_number = number % (2**index)
        if new_number == number:
            out += '0'
        else:
            out += '1'
            number = number - (2**index)
        index = index -1
    return out

